/*!
 * \file fw.h
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright ? 2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */


#include "..//include//al.h"
#include "..//include//chap_9.h"
#include "..//include//ci.h"
#include "..//include//usb100.h"


VOS_HANDLE hUart;
VOS_HANDLE hGpioOut;
VOS_HANDLE hFIFO;
VOS_HANDLE hGpioIn;

unsigned char ReadyForBulkIn=0;
unsigned char TimerExpiredForBulkIn=0;


unsigned char  keyPressed;

unsigned short num_read;
unsigned short num_written;
unsigned char  tmp=0;
unsigned short rx_count;

unsigned short buffer_size;
unsigned short num_bytes;
unsigned short wptr, rptr;
unsigned char  direction;
unsigned char  ep_stat;

//unsigned char  buffer1[6144];   // 6kB RAM max...
unsigned char  buffer1[SIZE_BUFFER1];

//unsigned char buffer2[512];

unsigned char  predict_iso_buf[128]={0};
unsigned long  passed=0;
unsigned long  failed=0;

unsigned short rd_count=0;
unsigned short wr_count=0;

unsigned long  byteCounterTx =0;
unsigned long  MbyteCounterTx=0;
unsigned long  byteCounterRx =0;
unsigned long  isoinByteCounter=0;
unsigned long  MbyteCounterRx=0;

extern unsigned long ClockIsr;

unsigned char *privateBuffer;
unsigned long dmaBuffer;
unsigned long ioBuffer; // V2.1
unsigned short ioSize, ioCount; // V2.1
unsigned char tempBuffer[130];

unsigned char isSpi=0;
unsigned char EnhancedModeEnabled=0;

unsigned char line_code[] =
{
0x00,	// baud rate: 0x0001C200 (115200)
0xC2,
0x01,
0x00,
0,		//FMT_1_STOP_BIT,	   // format
0,		//PARITY_NONE, 	   // parity
8		// num bits
};


/**************************************************************************
 *  Public static data
**************************************************************************/

extern EPPFLAGS bEPPflags;
extern volatile unsigned long ClockTicks;
extern unsigned char idata GenEpBuf[];
extern IO_REQUEST idata ioRequest;

CONTROL_XFER ControlData;


void help_devreq(unsigned char typ, unsigned char req)
{
	typ >>= 5;

	if(typ == USB_STANDARD_REQUEST)
	{
		//satya_20120116 printf("Request Type = %s, Request = %s.\n", _NAME_USB_REQUEST_TYPE[typ],_NAME_USB_STANDARD_REQUEST[req]);
	}
	else {
#ifdef __C51__
		if(bEPPflags.bits.verbose)
		{
			//satya_20120116 printf("Request Type = %s, bRequest = 0x%bx.\n", _NAME_USB_REQUEST_TYPE[typ],req);
		}
#else
		if(bEPPflags.bits.verbose)
		{
			//satya_20120116 printf("Request Type = %s, bRequest = 0x%x.\n", _NAME_USB_REQUEST_TYPE[typ],req);
		}
#endif
	}
}


void suspend_change(void)
{
}

void stall_ep0(void)
{
	FT12x_SetEndpointStatus(0, 1);
	FT12x_SetEndpointStatus(1, 1);
	DBG(MSG_DEBUG, "stall_ep0\r\n");

}

void disconnect_USB(void)
{
	// Initialize D12 configuration
	FT12x_SetMode(FT12X_NOLAZYCLOCK, FT12X_SETTOONE | FT12X_CLOCK_12M);
}

void connect_USB(void)
{
	// reset event flags
	DISABLE;
	bEPPflags.value = 0;
	ENABLE;

	// V2.1 enable normal+sof interrupt
	FT12x_SetDMA(FT12X_ENDP4INTENABLE | FT12X_ENDP5INTENABLE);
	FT12x_SetMode(FT12X_ENDP_NONISO|FT12X_SOFTCONNECT, FT12X_SETTOONE|FT12X_CLOCK_12M|FT12X_CLOCKRUNNING);
}

void reconnect_USB(void)
{
	unsigned long clk_cnt;
	
	disconnect_USB();

	clk_cnt = ClockTicks;
	while(ClockTicks < clk_cnt + 10);

	connect_USB();
}

void init_unconfig(void)
{
	FT12x_SetEndpointEnable(0);	/* Disable all endpoints but EPP0. */
}

void init_config(void)
{
	FT12x_SetEndpointEnable(1);	/* Enable  generic/iso endpoints. */
}

void single_transmit(unsigned char * buf, unsigned char len)
{
	if( len <= EP0_PACKET_SIZE) {
		FT12x_WriteEndpoint(1, buf, len);
	}
}

void code_transmit(unsigned char code * pRomData, unsigned short len)
{
	ControlData.wCount = 0;
	if(ControlData.wLength > len)
		ControlData.wLength = len;

	ControlData.pData = pRomData;
	if( ControlData.wLength >= EP0_PACKET_SIZE) {		
		FT12x_WriteEndpoint(1, ControlData.pData, EP0_PACKET_SIZE);
		ControlData.wCount += EP0_PACKET_SIZE;

		DISABLE;
		bEPPflags.bits.control_state = USB_TRANSMIT;
		ENABLE;
	}
	else {
		FT12x_WriteEndpoint(1, pRomData, ControlData.wLength);
		ControlData.wCount += ControlData.wLength;
		DISABLE;
		bEPPflags.bits.control_state = USB_IDLE;
		ENABLE;
	}
}

void control_handler()
{
	unsigned char type, req;

	type = ControlData.DeviceRequest.bmRequestType & USB_REQUEST_TYPE_MASK;
	req = ControlData.DeviceRequest.bRequest & USB_REQUEST_MASK;
	//printf("control_handler\n");

	help_devreq(type, req); // print out device request
	TDBG(MSG_DEBUG, "ENTER control_handler type=0x%x req=0x%x\r\n",(unsigned)type,(unsigned)req);

	if (type == USB_STANDARD_REQUEST)
	{
		DBG(MSG_DEBUG, "USB_STANDARD_REQUEST req=%u\r\n",(unsigned)req);
		switch(req)
		{
			case 0:  get_status();        break;
			case 1:  clear_feature();     break;
			case 3:  set_feature();       break;
			case 5:  set_address();       break;
			case 6:  get_descriptor();    break;
			case 8:  get_configuration(); break;
			case 9:  set_configuration(); break;
			case 10: get_interface();     break;
			case 11: set_interface();     break;
			default: reserved();          break;
		}
	}
	else if (USB_CLASS_REQUEST == type)
	{
		switch(ControlData.DeviceRequest.bRequest)
		{
			/*CDC-ACM requests*/
			case 0x21: //Get Line Coding
				single_transmit(line_code,sizeof(line_code));
				break;
			
			case 0x20: //Set Line Coding
			{
				int i;
				for(i=0;i<7;i++)
					line_code[i] = ControlData.dataBuffer[i];
				single_transmit(0, 0);
			}break;
			
			case 0x22: //Set Control Line State
				single_transmit(0, 0);
				break;

			default:
				reserved();
				break;

		}
	}
	else
		stall_ep0();
}
void remote_wakeup(void)
{

}


// Main Loop
void FT12x_main(void)
{

	unsigned char in_loop = 1 ;
	uint8 ledState = 0 ;
	unsigned short chipID=0;
	unsigned long ClockSec=0, last_clk=0;

	FT12x_Delay((unsigned long) 10000);
	chipID = FT12x_ReadChipID();
	if(chipID != 0x1012)
	{
		isSpi=1;
		SSP_IOConfig( SSP_NUM );	/* initialize SSP port, share pins with SPI1 on port2(p2.0-3). */
		SSP_Init( SSP_NUM );
		chipID = FT12x_ReadChipID();
#ifdef ENABLE_ENHANCED_MODE
		if(0x1012 == chipID)
			EnhancedModeEnabled = 1;
#endif
	}
	

	idProduct = FT12x_GetId(ID_CHIP);
	TDBG(MSG_DEBUG, "isSpi=%u ID_CHIP = 0x%x\r\n",isSpi, idProduct);
	idProduct = FT12x_GetId(ID_VENDOR);
	TDBG(MSG_DEBUG, "isSpi=%u ID_VENDOR = 0x%x\r\n",isSpi, idProduct);
	idProduct = FT12x_GetId(ID_PRODUCT);
	TDBG(MSG_DEBUG, "isSpi=%u ID_PRODUCT = 0x%x\r\n",isSpi, idProduct);	
	idProduct = FT12x_GetId(ID_FTDI);
	TDBG(MSG_DEBUG, "isSpi=%u ID_FTDI = 0x%x\r\n",isSpi, idProduct);
    reconnect_USB();

	/* Main program loop */
#ifdef TURN_OFF_LEDS
	Set_LED((uint8)0, 1);
	Set_LED((uint8)1, 1);
#endif

	while( in_loop )
	{
#ifdef PROCESS_ISR_IN_MAIN
		if(isr_pending)
		{
			DBG(MSG_DEBUG,"isr_pending=%u\n",isr_pending);
			fn_usb_isr();
			isr_pending--;
		}
#else
		if(0);
#endif
		else
		{
			fn_usb_isr();
			if (bEPPflags.bits.setup_packet)
			{
				DISABLE;
				bEPPflags.bits.setup_packet = 0;
				//ENABLE;
				DBG(MSG_DEBUG,"Setup packet - calling control_handler PSP=0x%x MSP=0x%x\r\n",(unsigned)__get_PSP,(unsigned)__get_MSP);
				control_handler();
				ENABLE;
			}
			else
			{
				if(bEPPflags.bits.setup_dma)
				{
					DISABLE;
					bEPPflags.bits.setup_dma = 0;
					ENABLE;

					DBG(MSG_DEBUG,"calling fill_ep2_buffer PSP=0x%x MSP=0x%x\r\n",(unsigned)__get_PSP,(unsigned)__get_MSP);

				} 
				else
				{

					{
						DISABLE;
						bEPPflags.bits.timer = 0;
						ENABLE;
#ifndef TURN_OFF_LEDS
						if(ClockTicks > (last_clk + 50))
						{
							last_clk = ClockTicks;		
							Set_LED((uint8)0, 1-ledState);
							Set_LED((uint8)1, ledState);
							Set_LED((uint8)2, 1-ledState);
							Set_LED((uint8)3, ledState);
							Set_LED((uint8)4, 1-ledState);
							ledState = ~ledState;
						}
#endif
					}
				}
			}
		}//if(isr_pending) else
	} // Main Loop
}

